﻿/**
* The Cyclone class.
* 
* @author Robert Penner (AS 1.0)
* © 2002 Robert Penner
* http://www.robertpenner.com
* 
* @author Mark Walters (AS 2.0)
* © 2006 DigitalFlipbook
* http://www.digitalflipbook.com
* 
* @version	1.0
* 
* @since	ActionScript 2.0; Flash Player 6
*/
class com.robertpenner.profmx.cyclone.Cyclone extends MovieClip {
	
	/*
	* ON STAGE
	*/
	private var p:MovieClip;
	private var p0:MovieClip;
	
	/*
	* CLASS PROPERTIES
	*/
	private var numParticles:Number;
	
	/*
	* CLIP PARAMETERS
	*/
	private var $clipParameters:Object = {maxParticles: 1, funnelHeight: 1, speed: 1, stiffness: 1};
	
	/*
	* INSPECTABLE PROPERTIES
	*/
	private var $maxParticles:Number;
	private var $funnelHeight:Number;
	private var $speed:Number;
	private var $stiffness:Number;
	
	
	/*
	* *********************************************************
	* CONSTRUCTOR
	* *********************************************************
	*
	*/
	
	function Cyclone () {
		this.initFromClipParameters();
	}
	
	
	/*
	* *********************************************************
	* PRIVATE METHODS
	* *********************************************************
	* 
	*/
	
	private function initFromClipParameters ():Void {
		var key:String, found:Boolean = false;
		for(key in $clipParameters) {
			if(this.hasOwnProperty(key)) {
				found = true;
				this["def_" + key] = this[key];
				delete this[key];
			}
		}
		if(found) {
			for(key in $clipParameters) {
				var v = this["def_" + key];
				if(v != undefined) {
					this[key] = v;
				}
			}
		}
	}
	
	private function init ():Void {
		this.p._visible = false;
		this.numParticles = 0;
		this.onEnterFrame = this.grow;
	}
	
	private function makeParticle (index) {
		var oval:MovieClip = this.attachMovie ("CycloneOval", "p" + index, index);
		var radius:Number = index*index / this.maxParticles + 2
		oval.path.radius = radius;
		with (oval.path.particle) {
			_xscale = _yscale = 60 + radius * .8;
		}
		oval._y = this.funnelHeight / (radius + 10);
	}
	
	private function grow () {
		if (this.numParticles < this.maxParticles) {
			this.makeParticle (this.maxParticles - this.numParticles);
			this.numParticles++;
		} else {
			delete this.onEnterFrame;
			this.p0.appear();
		}
	}
	
	private function sidewind () {
		for (var i=1; i <= this.maxParticles; i++) {
			this["p"+i].sidewind();
		}
	}
	
	
	/*
	* *********************************************************
	* PUBLIC METHODS
	* *********************************************************
	* 
	*/
	
	/**
	* Starts the sidewind of a Cyclone instance.
	* 
	* @usage
	* <pre><code>
	* cyclone.startSidewind();
	* </code></pre>
	*/
	public function startSidewind () {
		this.onEnterFrame = this.sidewind;
	}
	
	/**
	* Stops the sidewind of a Cyclone instance.
	* 
	* @usage
	* <pre><code>
	* cyclone.stopSidewind();
	* </code></pre>
	*/
	public function stopSidewind () {
		delete this.onEnterFrame;
	}
	
	
	/*
	* *********************************************************
	* GETTERS/SETTERS
	* *********************************************************
	* 
	*/
	
	/**
	* The maxParticles property of the Cyclone.
	* 
	* @usage
	* <pre><code>
	* trace (cyclone.maxParticles);
	* </code></pre>
	*/
	[Inspectable(type=Number, defaultValue=100)]
	public function get maxParticles ():Number {
		return this.$maxParticles;
	}
	public function set maxParticles (mp:Number):Void {
		 this.$maxParticles = mp;
	}
	
	/**
	* The funnelHeight property of the Cyclone.
	* 
	* @usage
	* <pre><code>
	* trace (cyclone.funnelHeight);
	* </code></pre>
	*/
	[Inspectable(type=Number, defaultValue=1500)]
	public function get funnelHeight ():Number {
		return this.$funnelHeight;
	}
	public function set funnelHeight (fh:Number):Void {
		 this.$funnelHeight = fh;
	}
	
	/**
	* The speed property of the Cyclone.
	* 
	* @usage
	* <pre><code>
	* trace (cyclone.speed);
	* </code></pre>
	*/
	[Inspectable(type=Number, defaultValue=300)]
	public function get speed ():Number {
		return this.$speed;
	}
	public function set speed (s:Number):Void {
		 this.$speed = s;
	}
	
	/**
	* The stiffness property of the Cyclone.
	* 
	* @usage
	* <pre><code>
	* trace (cyclone.stiffness);
	* </code></pre>
	*/
	[Inspectable(type=Number, defaultValue=.65)]
	public function get stiffness ():Number {
		return this.$stiffness;
	}
	public function set stiffness (s:Number):Void {
		 this.$stiffness = s;
	}
	
}
